module Sass
  module Rails
    autoload :Logger, 'sass/rails/logger'
  end
end

require 'sass/rails/version'
require 'sass/rails/helpers'
require 'sass/rails/importer'
require 'sass/rails/template'
require 'sass/rails/railtie'
